#!/usr/bin/env python
# -*- coding: utf-8 -*-
# taken from https://github.com/Elektordi/obs-websocket-py after running generate_classes.py to update the api

"""
Python library to communicate with an obs-websocket server.
"""

from .core import obsws  # noqa: F401



#!/usr/bin/env python3

try:
    import http.client as httplib
except ImportError:
    import httplib
import httplib2
import os
import sys
import errno
from time import sleep
from decimal import Decimal

from . import consts

from google_auth_oauthlib.flow import InstalledAppFlow

from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from googleapiclient.http import MediaFileUpload

from oauth2client.file import Storage
from oauth2client.tools import run_flow
from oauth2client.client import flow_from_clientsecrets

httplib2.RETRIES = 1

RETRIABLE_EXCEPTIONS = (
    httplib2.HttpLib2Error,
    IOError,
    httplib.NotConnected,
    httplib.IncompleteRead,
    httplib.ImproperConnectionState,
    httplib.CannotSendRequest,
    httplib.CannotSendHeader,
    httplib.ResponseNotReady,
    httplib.BadStatusLine,
)

RETRIABLE_STATUS_CODES = [500, 502, 504]

YOUTUBE_UPLOAD_SCOPE = "https://www.googleapis.com/auth/youtube.upload https://www.googleapis.com/auth/youtube https://www.googleapis.com/auth/youtube.readonly https://www.googleapis.com/auth/youtube.force-ssl"
YOUTUBE_PARTNER_SCOPE = "https://www.googleapis.com/auth/youtubepartner"
SPREADSHEETS_SCOPE = "https://www.googleapis.com/auth/spreadsheets"

PREFIXES = (
    consts.smash_folder,
    sys.prefix,
    os.path.join(sys.prefix, "local"),
    "/usr",
    os.path.join("/usr", "local"),
)
SUFFIXES = (
    "client_secrets.json",
    ".client_secrets.json",
    f"share/{consts.short_name}/client_secrets.json",
)


def upload(yt, body, file, notify=False):
    vid = None
    ret = None
    retries = 0
    while not vid and retries < 10:
        insert_request = yt.videos().insert(
            part=",".join(body.keys()),
            body=body,
            notifySubscribers=notify,
            media_body=MediaFileUpload(file, chunksize=104857600, resumable=True),
        )
        ret, vid = upload_service(insert_request)
        retries += 1
    return ret, vid


def upload_service(insert_request):
    response = None
    retry_exceptions = RETRIABLE_EXCEPTIONS
    retry_status_codes = RETRIABLE_STATUS_CODES
    ACCEPTABLE_ERRNO = (errno.EPIPE, errno.EINVAL, errno.ECONNRESET)
    try:
        ACCEPTABLE_ERRNO += (errno.WSAECONNABORTED,)
    except AttributeError:
        pass  # Not windows
    while True:
        try:
            status, response = insert_request.next_chunk()
            if status is not None:
                percent = Decimal(
                    int(status.resumable_progress) / int(status.total_size)
                )
                print(f"{round(100 * percent, 2)}% uploaded")
        except HttpError as e:
            if e.resp.status in retry_status_codes:
                print(f"A retriable HTTP error {e.resp.status} occurred:\n{e.content}")
            elif b"503" in e.content:
                print("Backend Error: will attempt to retry upload")
                return False, None
            elif b"uploadLimitExceeded" in e.content:
                print("You have exceeded the YouTube Upload Limit")
                print("Waiting 10 minutes before retrying to avoid the limit")
                sleep(600)
            else:
                print(e)
                return False, None
        except retry_exceptions as e:
            print(f"A retriable error occurred: {e}")

        except Exception as e:
            if e in ACCEPTABLE_ERRNO:
                print("Retriable Error occured, retrying now")
            else:
                print(e)
            pass
        if response:
            video_id = response.get("id", None)
            if video_id is None:
                print(response)
                print(status)
                return False, None
            print(f"Video link is\nhttps://www.youtube.com/watch?v={video_id}")
            return True, video_id


def test_get_service(scope, oauth_file, secret=None):
    """
    WIP
    Based on the newer google_auth_oauthlib module
    """
    CLIENT_SECRETS_FILE = get_secrets(PREFIXES, SUFFIXES) if not secret else secret

    if not CLIENT_SECRETS_FILE:
        return None

    flow = InstalledAppFlow.from_client_secrets_file(CLIENT_SECRETS_FILE, scopes=scope)

    storage = Storage(oauth_file)
    credentials = storage.get()

    if credentials is None or credentials.invalid:
        credentials = flow.run_local_server(
            host="localhost",
            port=8080,
            authorization_prompt_message="Please visit this URL: {url}",
            success_message="The auth flow is complete; you may close this window.",
            open_browser=True,
        )
        storage.put(credentials)

    return credentials


def get_service(scope, oauth_file, secret=None):
    CLIENT_SECRETS_FILE = get_secrets(PREFIXES, SUFFIXES) if not secret else secret

    if not CLIENT_SECRETS_FILE:
        return None

    flow = flow_from_clientsecrets(CLIENT_SECRETS_FILE, scope=scope)

    flow.user_agent = consts.long_name
    storage = Storage(oauth_file)
    credentials = storage.get()

    if credentials is None or credentials.invalid:
        credentials = run_flow(flow, storage)

    return credentials


def get_youtube_service():
    credentials = get_service(YOUTUBE_UPLOAD_SCOPE, consts.youtube_oauth_file)

    if not credentials:
        return None

    http = httplib2.Http()
    try:
        http.redirect_codes = set(http.redirect_codes) - {
            308
        }  # https://github.com/googleapis/google-api-python-client/issues/803
    except:
        pass

    return build("youtube", "v3", http=credentials.authorize(http))


def get_partner_service():
    CLIENT_SECRETS_FILE = get_secrets(
        (consts.smash_folder,), ("client_secrets.json", ".client_secrets.json")
    )

    credentials = get_service(
        YOUTUBE_PARTNER_SCOPE + YOUTUBE_UPLOAD_SCOPE, "partner", CLIENT_SECRETS_FILE
    )

    if not credentials:
        return None

    http = httplib2.Http()
    try:
        http.redirect_codes = set(http.redirect_codes) - {
            308
        }  # https://github.com/googleapis/google-api-python-client/issues/803
    except:
        pass

    return build("youtubePartner", "v1", http=credentials.authorize(http))


def get_spreadsheet_service():
    credentials = get_service(SPREADSHEETS_SCOPE, "spreadsheet")

    if not credentials:
        return None

    discoveryUrl = "https://sheets.googleapis.com/$discovery/rest?version=v4"

    http = httplib2.Http()
    try:
        http.redirect_codes = set(http.redirect_codes) - {
            308
        }  # https://github.com/googleapis/google-api-python-client/issues/803
    except:
        pass

    return build(
        "sheets",
        "v4",
        http=credentials.authorize(http),
        discoveryServiceUrl=discoveryUrl,
    )


def add_to_playlist(pID, vID):
    consts.youtube.playlistItems().insert(
        part="snippet",
        body=dict(
            snippet=dict(
                playlistId=pID, resourceId=dict(kind="youtube#video", videoId=vID)
            )
        ),
    ).execute()
    print("Added to playlist")


def get_secrets(prefixes, relative_paths):
    """
    Taken from https://github.com/tokland/youtube-upload/blob/master/youtube_upload/main.py
    Get the first existing filename of relative_path seeking on prefixes directories.
    """
    paths_attempted = []
    try:
        return os.path.join(sys._MEIPASS, relative_paths[-1])
    except Exception:
        for prefix in prefixes:
            for relative_path in relative_paths:
                path = os.path.join(prefix, relative_path)
                if os.path.exists(path):
                    print(f"found client_secrets.json at {path}")
                    return path
                paths_attempted.append(path)
        else:
            print(
                f"Unable to find client_secrets.json. Checked in the following locations: {paths_attempted}"
            )
            return None



#!/usr/bin/env python3
from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))
version = "1.21.6"

long_des = ""
with open(path.join(here, "README.md")) as f:
    long_des = f.read()

setup(
    name="MeleeUploader",
    description="A YouTube Uploader with Super Smash Bros. in mind",
    long_description=long_des,
    long_description_content_type="text/markdown",
    url="https://github.com/NikhilNarayana/Melee-YouTube-Uploader",
    author="Nikhil Narayana",
    author_email="nikhil.narayana@live.com",
    license="GNU Public License v3.0",
    keywords="smash melee ssbm ssbu ultimate youtube uploader",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
    ],
    entry_points=dict(
        console_scripts=[
            "meleeuploader = meleeuploader.main:main",
            "smashuploader = meleeuploader.main:ult",
            "s64uploader = meleeuploader.main:s64",
            "rivalsuploader = meleeuploader.main:rivals",
            "splatoonuploader = meleeuploader.main:splatoon",
        ]
    ),
    python_requires=">= 3.7.0",
    version=version,
    packages=["meleeuploader", "meleeuploader.obswebsocket"],
    install_requires=[
        "CacheControl",
        "google-api-python-client",
        "google_auth_oauthlib",
        "oauth2client",
        "Pyforms-Lite",
        "urllib3==1.23",
        "websocket-client==0.54.0",
    ],
    data_files=[("share/meleeuploader", ["meleeuploader/client_secrets.json"])],
    package_data={"meleeuploader": ["meleeuploader/client_secrets.json"]},
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64
import hashlib
import json
import logging
import socket
import threading
import time
import websocket

from . import exceptions
from . import base_classes
from . import events

LOG = logging.getLogger(__name__)


class obsws:
    """
    Core class for using obs-websocket-py

    Simple usage:
        >>> import obswebsocket, obswebsocket.requests as obsrequests
        >>> client = obswebsocket.obsws("localhost", 4444, "secret")
        >>> client.connect()
        >>> client.call(obsrequests.GetVersion()).getObsWebsocketVersion()
        u'4.1.0'
        >>> client.disconnect()

    For advanced usage, including events callback, see the 'samples' directory.
    """

    def __init__(self, host="localhost", port=4444, password=""):
        """
        Construct a new obsws wrapper

        :param host: Hostname to connect to
        :param port: TCP Port to connect to (Default is 4444)
        :param password: Password for the websocket server (Leave this field
            empty if no auth enabled on the server)
        """
        self.id = 1
        self.thread_recv = None
        self.ws = None
        self.eventmanager = EventManager()
        self.answers = {}

        self.host = host
        self.port = port
        self.password = password

    def connect(self, host=None, port=None):
        """
        Connect to the websocket server

        :return: Nothing
        """
        if host is not None:
            self.host = host
        if port is not None:
            self.port = port

        try:
            LOG.info("Connecting...")
            self.ws = websocket.WebSocket()
            self.ws.connect((f"ws://{self.host}:{self.port}"))
            LOG.info("Connected!")
            self._auth(self.password)
            self._run_threads()
        except socket.error as e:
            raise exceptions.ConnectionFailure(str(e))

    def reconnect(self):
        """
        Restart the connection to the websocket server

        :return: Nothing
        """
        try:
            self.disconnect()
        except Exception:
            # TODO: Need to catch more precise exception
            pass
        self.connect()

    def disconnect(self):
        """
        Disconnect from websocket server

        :return: Nothing
        """
        LOG.info("Disconnecting...")
        if self.thread_recv is not None:
            self.thread_recv.running = False

        try:
            self.ws.close()
        except socket.error:
            pass

        if self.thread_recv is not None:
            self.thread_recv.join()
            self.thread_recv = None

    def _auth(self, password):
        auth_payload = {
            "request-type": "GetAuthRequired",
            "message-id": str(self.id),
        }
        self.id += 1
        self.ws.send(json.dumps(auth_payload))
        result = json.loads(self.ws.recv())

        if result["status"] != "ok":
            raise exceptions.ConnectionFailure(result["error"])

        if result.get("authRequired"):
            secret = base64.b64encode(
                hashlib.sha256((password + result["salt"]).encode("utf-8")).digest()
            )
            auth = base64.b64encode(
                hashlib.sha256(secret + result["challenge"].encode("utf-8")).digest()
            ).decode("utf-8")

            auth_payload = {
                "request-type": "Authenticate",
                "message-id": str(self.id),
                "auth": auth,
            }
            self.id += 1
            self.ws.send(json.dumps(auth_payload))
            result = json.loads(self.ws.recv())
            if result["status"] != "ok":
                raise exceptions.ConnectionFailure(result["error"])
        pass

    def _run_threads(self):
        if self.thread_recv is not None:
            self.thread_recv.running = False
        self.thread_recv = RecvThread(self)
        self.thread_recv.daemon = True
        self.thread_recv.start()

    def call(self, obj):
        """
        Make a call to the OBS server through the Websocket.

        :param obj: Request (class from obswebsocket.requests module) to send
            to the server.
        :return: Request object populated with response data.
        """
        if not isinstance(obj, base_classes.Baserequests):
            raise exceptions.ObjectError("Call parameter is not a request object")
        payload = obj.data()
        r = self.send(payload)
        obj.input(r)
        return obj

    def send(self, data):
        """
        Make a raw json call to the OBS server through the Websocket.

        :param data: Request (python dict) to send to the server. Do not
            include field "message-id".
        :return: Response (python dict) from the server.
        """
        message_id = str(self.id)
        self.id += 1
        data["message-id"] = message_id
        LOG.debug(f"Sending message id {message_id}: {data}")
        self.ws.send(json.dumps(data))
        return self._wait_message(message_id)

    def _wait_message(self, message_id):
        timeout = time.time() + 60  # Timeout = 60s
        while time.time() < timeout:
            if message_id in self.answers:
                return self.answers.pop(message_id)
            time.sleep(0.1)
        raise exceptions.MessageTimeout(f"No answer for message {message_id}")

    def register(self, func, event=None):
        """
        Register a new hook in the websocket client

        :param func: Callback function pointer for the hook
        :param event: Event (class from obswebsocket.events module) to trigger
            the hook on. Default is None, which means trigger on all events.
        :return: Nothing
        """
        self.eventmanager.register(func, event)

    def unregister(self, func, event=None):
        """
        Unregister a new hook in the websocket client

        :param func: Callback function pointer for the hook
        :param event: Event (class from obswebsocket.events module) which
            triggered the hook on. Default is None, which means unregister this
            function for all events.
        :return: Nothing
        """
        self.eventmanager.unregister(func, event)


class RecvThread(threading.Thread):
    def __init__(self, core):
        self.core = core
        self.ws = core.ws
        self.running = True
        threading.Thread.__init__(self)

    def run(self):
        while self.running:
            message = ""
            try:
                message = self.ws.recv()

                # recv() can return an empty string (Issue #6)
                if not message:
                    continue

                result = json.loads(message)
                if "update-type" in result:
                    LOG.debug(f"Got message: {result}")
                    obj = self.build_event(result)
                    self.core.eventmanager.trigger(obj)
                elif "message-id" in result:
                    LOG.debug(f"Got answer for id {result['message-id']}: {result}")
                    self.core.answers[result["message-id"]] = result
                else:
                    LOG.warning(f"Unknown message: {result}")
            except websocket.WebSocketConnectionClosedException:
                if self.running:
                    self.core.reconnect()
            except OSError as e:
                if self.running:
                    raise e
            except (ValueError, exceptions.ObjectError) as e:
                LOG.warning(f"Invalid message: {message} ({e})")
        # end while
        LOG.debug("RecvThread ended.")

    @staticmethod
    def build_event(data):
        name = data["update-type"]
        try:
            obj = getattr(events, name)()
        except AttributeError:
            raise exceptions.ObjectError(f"Invalid event {name}")
        obj.input(data)
        return obj


class EventManager:
    def __init__(self):
        self.functions = []

    def register(self, callback, trigger):
        self.functions.append((callback, trigger))

    def unregister(self, callback, trigger):
        for c, t in self.functions:
            if (c == callback) and (trigger is None or t == trigger):
                self.functions.remove((c, t))

    def trigger(self, data):
        for callback, trigger in self.functions:
            if trigger is None or isinstance(data, trigger):
                callback(data)
